/* Electronic Voting Project
 * Complaint Sender
 */

package evote.counter;

import evote.network.server.*;
import evote.network.message.*;
import evote.util.constants.*;
import evote.util.genrand.*;
import evote.commissioner.complaints.*;
import evote.crypto.*;

public class ComplaintSender {
  
  private boolean success= false;

  public ComplaintSender(Complaint c, GenRand gr, Signer s, Server serv) {
    
    /* Create the RSA Keys for the Commissioner */
    RsaKeys rsk= new RsaKeys(Constants.commisPublic, Constants.commisMod);

    /* Create the Connection */
    try {
      SecureConnection sc= new SecureConnection(Constants.commisServer, Constants.commisReceivePort, gr, rsk);
      sc.sendMessage(new ComplaintMessage(c, s));
      sc.close();
    }
    catch (Exception e) {
      success= false;
      serv.log(new LogEntry(LogEntry.SERIOUS_LOG,"Could not contact Commissioner to send Complaint:\n " + c.getProblemString()));
      return;
    }
    
    serv.log(new LogEntry(LogEntry.WARNING_LOG,"Sent complaint to Commissioner:\n" + c.getProblemString()));
    success= true;
  }

  public boolean wasSuccessful() {
    return(success);
  }
}
