
/*
 * This class contains the results for one question
 */

package evote.counter;

import java.io.*;
import java.util.*;

import evote.question.*;
import evote.lang.*;


public class Result implements Serializable {
  
    private Question myQuestion;

    // This array of counts contains both real choices
    // and write-ins
    private Vector counts;
    private OneCount dummy;
  
    public Result(Question q) {
	myQuestion= q;

	counts= new Vector();
    
	// Get all the answers
	String[] ans= myQuestion.getAnswers();
	for (int i=0; i<ans.length; i++) 
	    counts.addElement(new OneCount(ans[i], 0));

	dummy= new OneCount("",0);
    }

    public void incrementChoice(FullChoice c, int ch) {
	// if no choice selected, then stop
	if (ch == -1)
	    return;

	((OneCount)counts.elementAt(ch)).increment();
	((OneCount)counts.elementAt(ch)).addChoice(c);
    }

    public void incrementWriteIn(FullChoice c, String str) {
	// since we do .equals() on OneCounts only by the
	// answer String, we can find the answer with the
	// dummy String
	dummy.answer= str.toUpperCase();
	int which_one= counts.lastIndexOf(dummy);

	// If we find the OneCount, then increment it.
	// otherwise, create a new count.
	if (which_one>-1) {
	    this.incrementChoice(c, which_one);
	} else {
	    OneCount newCount= new OneCount(str.toUpperCase(), 1);
	    newCount.addChoice(c);
	    counts.addElement(newCount);
	}
    }

    private OneCount getOneCount(Choice c, int question_num) throws IndexOutOfBoundsException {
	try {
	    // try finding the Choice
	    int which_one= c.getChoice(question_num);
	    return ((OneCount)counts.elementAt(which_one));
	} catch (WriteInException wie) {
	    try {
		// try finding the write in
		String which_str= c.getWriteInChoice(question_num);
		
		// use the dummy to find this
		dummy.answer= which_str.toUpperCase();
		int which_one= counts.lastIndexOf(dummy);
		
		if (which_one == -1)
		    return(null);
		
		return((OneCount)counts.elementAt(which_one));
	    } catch (ChoiceException exc) {
		return(null);
	    }
	}
    }
	
    public void consolidate(int[] to_consolidate, String newName) {
	if (to_consolidate.length < 2)
	    return;

	Vector toDelete= new Vector();

	OneCount sum= (OneCount)counts.elementAt(to_consolidate[0]);
	toDelete.addElement(sum);
    
	for (int i=1; i<to_consolidate.length; i++) {
	    OneCount newEl= (OneCount)counts.elementAt(to_consolidate[i]);
	    toDelete.addElement(newEl);
	    sum= OneCount.merge(sum, newEl, newName);
	}

	Enumeration enum= toDelete.elements();
	while(enum.hasMoreElements()) {
	    counts.removeElement(enum.nextElement());
	}

	counts.addElement(sum);
    }

    public void delete(int pos) {
	// get the choices to downgrade,
	// and immediately mark that count 
	// deleted, because we don't need it anymore.
	Vector votesToDowngrade= ((OneCount)counts.elementAt(pos)).choices;
	Enumeration enum= votesToDowngrade.elements();
	((OneCount)counts.elementAt(pos)).delete();

	// We now have the list of choices in enum
	// that need to be downgraded to their next choice

	FullChoice fc;
	int counter;

	// we loop through all the choices to delete
	while (enum.hasMoreElements()) {
	
	    // get the Full Choice, which includes the questionlist
	    // of the particular choice, because a choice object
	    // is relative to its questionlist
	    fc= (FullChoice)enum.nextElement();
	    
	    // a quick loop to find the Question that this Choice answers
	    // we use the question ID and questionlist ID.
	    for (counter=0; counter<fc.ql.getNumQuestions(); counter++) {
		if (myQuestion.getQuestionID() == fc.ql.getQuestion(counter).getQuestionID())
		    if (myQuestion.getQuestionListID() == fc.ql.getQuestion(counter).getQuestionListID())
			break;
	    }
      
	    // downgrade the question
	    // at position "counter" in the Choice
	    // until we get to a non-deleted choice!
	    try {
		// get the OneCount we want.
		OneCount currentCount= getOneCount(fc.choice, counter);
		
		// keep downgrading until we find a non-deleted,
		// or new OneCount
		while(true) {
		    // if it doesn't exist yet
		    if (currentCount == null)
			break;
		    
		    // if it's not deleted
		    if (!currentCount.isDeleted())
			break;
		    
		    ((MultipleChoice)fc.choice).downgradeQuestion(counter);
		    currentCount= getOneCount(fc.choice, counter);
		}
	    } catch (IndexOutOfBoundsException exc) {
		// do nothing, the vote is just now too downgraded,
		// so we just keep going.
	    }

	    // recount that vote, now that it's been downgraded.
	    try {
		try {
		    // first we try incrementing the choice
		    incrementChoice(fc, fc.choice.getChoice(counter));
		} catch(WriteInException wie) {
		    try {
			// we try incrementing the write in
			incrementWriteIn(fc, fc.choice.getWriteInChoice(counter));
		    } catch(ChoiceException ce) {
		    }
		}
	    } catch(IndexOutOfBoundsException iobe) {
		// an error happened, meaning this vote now has
		// no more preferences to give us.
	    }
    
	}

    }

    public Question getQuestion() {
	return(myQuestion);
    }

    public int getNumAnswers() {
	return(counts.size());
    }

    public boolean isDeleted(int i) {
	return(((OneCount)counts.elementAt(i)).deleted_p);
    }

    public String getAnswer(int i) {
	return(((OneCount)counts.elementAt(i)).answer);
    }

    public int getCount(int i) {
	return(((OneCount)counts.elementAt(i)).count);
    }

    public String toString() {
	Enumeration enum= counts.elements();
	OneCount oc;
	String returnString= "";

	while(enum.hasMoreElements()) {
	    oc= (OneCount)enum.nextElement();
	    returnString+= oc.answer + "-" + oc.count + "\n";
	}

	return(returnString);
    }
}

class OneCount implements Serializable {
    protected String answer;
    protected int count;
    protected Vector choices;

    protected boolean deleted_p= false;

    protected OneCount(String a, int i) {
	answer= a;
	count= i;
	choices= new Vector();
    }

    protected static OneCount merge(OneCount first, OneCount second, String newName) {
	OneCount returnCount= new OneCount(newName, first.count+second.count);
	Enumeration enum;

	enum= first.choices.elements();
	while (enum.hasMoreElements())
	    returnCount.addChoice((FullChoice)enum.nextElement());

	enum= second.choices.elements();
	while (enum.hasMoreElements())
	    returnCount.addChoice((FullChoice)enum.nextElement());

	return(returnCount);
    }

    protected void delete() {
	deleted_p= true;
	choices.removeAllElements();
    }

    protected boolean isDeleted() {
	return(deleted_p);
    }

    protected synchronized void increment() {
	count+= 1;
    }

    protected void addChoice(FullChoice c) {
	if (!deleted_p)
	    choices.addElement(c);
    }

    // This method compares only the String,
    // so that a count is considered the same as 
    // another if their name is the same.
    public boolean equals(OneCount oc) {
	return(this.answer.equals(oc.answer));
    }
}
