
/*
 * A GUI for parsing results
 */

package evote.counter;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import evote.lang.*;
import evote.question.*;

public class ResultInterface extends Frame implements ActionListener {
  
  private ResultSet[] results;

  private List resultList;
  private Button consolidateButton, deleteButton;
  private TextField newNameField;
  private MenuBar myMenuBar;
  private Menu[] menus;
  private MenuItem[][] submenus;

  private int currentResultSet=-1;
  private int currentResult=-1;
  
  public ResultInterface(ResultSet[] res) {
    super("Results");

    results= res;

    this.setLayout(new BorderLayout());

    resultList=new List();
    resultList.setMultipleMode(true);

    this.add("Center",resultList);
    
    Panel consolidatePanel= new Panel(new FlowLayout());
    newNameField= new TextField(25);
    consolidateButton= new Button("Consolidate");
    consolidateButton.addActionListener(this);
    consolidatePanel.add(newNameField);
    consolidatePanel.add(consolidateButton);
    this.add("North", consolidatePanel);
    
    Panel deletePanel= new Panel(new FlowLayout());
    deleteButton= new Button("Delete");
    deleteButton.addActionListener(this);
    deletePanel.add(deleteButton);
    this.add("South",deletePanel);

    myMenuBar= new MenuBar();
    menus= new Menu[results.length];
    submenus= new MenuItem[results.length][];
    Menu mainMenu= new Menu("Results");

    for (int i=0; i<results.length; i++) {
      menus[i]= new Menu("ResultSet #" + i);
      submenus[i]= new MenuItem[results[i].getNumResults()];
      for (int j=0; j<submenus[i].length; j++) {
	submenus[i][j]= new MenuItem("Result #" + j);
	submenus[i][j].addActionListener(this);
	menus[i].add(submenus[i][j]);
      }
      mainMenu.add(menus[i]);
    }
    
    myMenuBar.add(mainMenu);
 
    this.setMenuBar(myMenuBar);
    this.setSize(400,600);
    // this.pack();
    this.show();
  }

  public static void main(String argv[]) {
    try {
      FileInputStream fis= new FileInputStream(argv[0]);
      ObjectInputStream ois= new ObjectInputStream(fis);

      ResultSet[] in= (ResultSet[])ois.readObject();

      ResultInterface rif= new ResultInterface(in);
    } catch (Exception exc) {
      exc.printStackTrace();
    }
  }
  
  private void reset() {
    resultList.removeAll();
    
    Result thisResult= results[currentResultSet].getResult(currentResult);
    for (int i=0; i<thisResult.getNumAnswers(); i++) {
      if (!thisResult.isDeleted(i))
	resultList.add(thisResult.getAnswer(i)+ " -- " + thisResult.getCount(i), i);
      else
	resultList.add(thisResult.getAnswer(i)+ " -- XXXXX");
    }
  }

  public void actionPerformed(ActionEvent ae) {
    Object source= ae.getSource();
    
    if (source == deleteButton) {
      int[] selected= resultList.getSelectedIndexes();
      if (selected.length != 1)
	return;

      results[currentResultSet].getResult(currentResult).delete(selected[0]);
      reset();
      return;
    }
	
    if (source == consolidateButton) {
      results[currentResultSet].getResult(currentResult).consolidate(resultList.getSelectedIndexes(), newNameField.getText());
      reset();
      return;
    }
    
    for (int i=0; i<menus.length; i++) {
      for (int j=0; j<submenus[i].length; j++) {
	if (submenus[i][j] == source) {
	  currentResultSet= i;
	  currentResult= j;
	  reset();
	}
      }
    }

  }
}

class OneChoice {
  protected evote.lang.Choice choice;
  protected QuestionList ql;

  protected OneChoice(evote.lang.Choice ch, QuestionList q) {
    choice= ch;
    ql= q;
  }
}
