/* Electronic Voting Project
 * The Vote Counter Display
 * Ben Adida (ben@mit.edu)
 */

package evote.counter;

import java.awt.*;
import java.awt.event.*;
import evote.lang.*;
import evote.counter.*;
import evote.election.*;
import evote.question.*;
import java.util.Vector;
import java.util.Enumeration;

/**
 * The Vote Counter Display
 *
 * This is a simple GUI that displays the results of the Vote Count
 *
 * @see VoteCounter
 * @see Counter
 */
public class VoteCounterDisplay extends Frame implements WindowListener, ActionListener {
  private Election my_election;
  private int[][][] vote_counts;
  private Vector[][] write_ins;

  private ButtonHandler back_handler, next_handler;

  private CardLayout my_layout;

  /**
   * Create a new Vote Counter Display
   *
   * @argument e The Election object
   * @argument vc The 2-dimensional array of counts to each answer of each question.
   * @argument w The array of vectors of write-ins for each question
   */
  public VoteCounterDisplay(Election e, int[][][] vc, Vector[][] w) {
    my_election= e;
    vote_counts= vc;
    write_ins= w;
    
    back_handler= new ButtonHandler(this,false);
    next_handler= new ButtonHandler(this,true);

    this.addWindowListener(this);
    
    my_layout= new CardLayout();
    this.setLayout(my_layout);

    QuestionList[] questionLists= my_election.getQuestionLists();
    Question currentQuestion;
    for (int i=0; i<vc.length; i++) {
      for (int questCounter=0; questCounter<vc[i].length; questCounter++) {
	currentQuestion= questionLists[i].getQuestion(questCounter);

	String[] answ= currentQuestion.getAnswers();
	String[] wi= new String[write_ins[i][questCounter].size()];
	for (int j=0; j<wi.length; j++) 
	  wi[j]= (String)write_ins[i][questCounter].elementAt(j);
	this.add("" + i + "," + questCounter, new QuestionPanel(currentQuestion.getQuestion(), answ, vc[i][questCounter], wi, back_handler, next_handler));
      }
    }

    // System.out.println("ABOUT to put up the Vote Counter window");

    this.pack();
    this.resize(400,300);
    this.show();
  }

  protected void nextQuestion() {
    my_layout.next(this);
  }
  
  protected void previousQuestion() {
    my_layout.previous(this);
  }

  public void actionPerformed(ActionEvent ae) {
  }
     
  public void windowActivated(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }
  
  public void windowClosing(WindowEvent e) {
    this.dispose();
  }

  public void windowDeactivated(WindowEvent e) {
  }

  public void windowDeiconified(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
  }

  public void windowOpened(WindowEvent e) {
  }



}

class ButtonHandler implements ActionListener {
  private VoteCounterDisplay vcd;
  boolean is_next;

  public ButtonHandler(VoteCounterDisplay v, boolean is_n) {
    vcd= v;
    is_next= is_n;
  }
  
  public void actionPerformed(ActionEvent ae) {
    if (is_next)
      vcd.nextQuestion();
    else
      vcd.previousQuestion();
  }
}


class QuestionPanel extends Panel {
  private String question;
  private String[] answers;
  private int[] answer_counts;
  private String[] write_ins;

  private Button back_button, next_button;

  public QuestionPanel(String q, String[] a, int[] a_c, String[] wi, ActionListener back_handler, ActionListener next_handler) {
    super();

    this.question= q;
    this.answers= a;
    this.answer_counts= a_c;
    this.write_ins= wi;

    this.setLayout(new BorderLayout());
    this.add("North",new Label(question));

    Panel center_p= new Panel();
    center_p.setLayout(new GridLayout(1,2));
    
    Panel center_left= new Panel();
    center_left.setLayout(new GridLayout(a.length,1));

    for (int i=0; i<a.length; i++)
      center_left.add(new Label("" + answer_counts[i] + " - " + answers[i]));
    
    Panel center_right= new Panel();
    center_right.setLayout(new BorderLayout());
    
    Panel center_right_top= new Panel();
    center_right_top.setLayout(new GridLayout(1,2));
    center_right_top.add(new Label("# Write-Ins"));
    TextField tf= new TextField();
    tf.setEditable(false);
    tf.setText("" + write_ins.length);
    center_right_top.add(tf);

    center_right.add("North",center_right_top);

    List write_in_list= new List();
    for (int i=0; i<write_ins.length; i++)
      write_in_list.addItem(write_ins[i]);

    center_right.add("Center",write_in_list);

    center_p.add(center_left);
    center_p.add(center_right);

    this.add("Center",center_p);

    back_button= new Button("back");
    next_button= new Button("next");

    Panel bottom_p= new Panel();
    bottom_p.add(back_button);
    bottom_p.add(next_button);

    this.add("South",bottom_p);
    
    back_button.addActionListener(back_handler);
    next_button.addActionListener(next_handler);
  }

}
