/* Electronic Voting Project
 * Mark A. Herschberg (hershey@mit.edu)
 */

package evote.crypto;

/**
  Interface specifying the methods need to commit.
  */
public interface Commitment {
  
  /**
    The class must have a method to commit.
    */
  public byte[] commit(byte[] input);
  
  /**
    The class must have a method to uncommit which throws an exception if it is
    unable to uncommit the input (say due to a lack of keys).
    */
  public byte[] uncommit(byte[] b) throws Exception;
  
  /**
    The class must have a method to return the keys used to commit.
    */
  public CommitmentKeys getKeys();
  
}
