/*
 * This class is a demo interface, which means that it contains one big 
 * text display that can be used to describe status, and a count, to
 * give some knowledge of progress
 */

package evote.demo;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class DemoInterface extends Frame {
    
    private static final int WIDTH=600;
    private static final int HEIGHT=400;
    private static final int PAUSE= 3500;

    private String currentStatus;
    private int currentCount;

    private Font myFont= new Font("Helvetica", Font.BOLD, 36);
    private FontMetrics myFontMetrics;

    public DemoInterface(String name) {
	super(name);

	currentStatus="";
	currentCount=0;

	this.pack();
	this.setSize(WIDTH, HEIGHT);
	this.show();
	this.clearStatus();
    }

    // change the status message
    public void setStatus(String newStatus) {

	//System.err.println("new status: " + newStatus);
	currentStatus= newStatus;
	update();

	try {
	    Thread.sleep(PAUSE);
	} catch (InterruptedException exc) {
	}
    }

    public void clearStatus() {
	//System.err.println("clear status");
	currentStatus= "Count: " + currentCount;
	update();
    }
	    
    // increment the count
    public void incrementCount() {
	synchronized(this) {
	    currentCount+= 1;
	}
	update();
    }

    public void paint(Graphics g) {
	// split string along newlines
	StringTokenizer stok= new StringTokenizer(currentStatus,"\n");
	String[] lines= new String[stok.countTokens()];
	int i=0;
	while (stok.hasMoreTokens())
	    lines[i++]= stok.nextToken();
	
	if (myFontMetrics == null) {
	    myFontMetrics= g.getFontMetrics(myFont);
	}

	g.setColor(Color.white);
	g.fillRect(0,0,WIDTH,HEIGHT);
	g.setColor(Color.black);

	int start_height= (HEIGHT - (lines.length * myFontMetrics.getHeight()))/2;
	int start_width;

	g.setFont(myFont);
	for (int counter=0; counter<lines.length; counter++) {
	    start_width= (WIDTH - myFontMetrics.stringWidth(lines[counter]))/2;

	    g.drawString(lines[counter], start_width, start_height);
	    start_height+= myFontMetrics.getHeight();
	}
    }

    public void update() {
	paint(this.getGraphics());
    }
   
    public void repaint() {
	paint(this.getGraphics());
    }

    public void destroy() {
	this.dispose();
    }
}
