/*
 * Time Setup Panel
 * This sets up voting times
 */

package evote.election.builder;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

import evote.lang.*;
import evote.election.*;
import evote.counter.*;
import evote.admin.*;
import evote.anonymizer.*;
import evote.commissioner.*;

public class TimeSetupPanel extends Panel implements ActionListener {

  /* The server params */
  private CounterParameters counterParams;
  private AdminParameters adminParams;
  private AnonymizerParameters anonymizerParams;
  private CommissionerParameters commissionerParams;

  /* GUI Elements */
  private TimePanel startAdminTimePanel, endAdminTimePanel, startAnonymizerTimePanel, endAnonymizerTimePanel, startCounterTimePanel, endCounterTimePanel, startCommissionerTimePanel, endCommissionerTimePanel;

  // commit buttons
  private Button commitAdminStartButton, commitAdminEndButton, commitAnonymizerStartButton, commitAnonymizerEndButton, commitCounterStartButton, commitCounterEndButton, commitCommissionerStartButton, commitCommissionerEndButton;

  public TimeSetupPanel(CounterParameters count_params, 
			AdminParameters admin_params, 
			AnonymizerParameters anon_params,
			CommissionerParameters commis_params) {

    // set up stuff
    counterParams= count_params;
    adminParams= admin_params;
    anonymizerParams= anon_params;
    commissionerParams= commis_params;
    
    // time panels 
    startAdminTimePanel= new TimePanel(admin_params.getStart());
    endAdminTimePanel= new TimePanel(admin_params.getEnd());
    startAnonymizerTimePanel= new TimePanel(anon_params.getStart());
    endAnonymizerTimePanel= new TimePanel(anon_params.getEnd());
    startCounterTimePanel= new TimePanel(count_params.getStart());
    endCounterTimePanel= new TimePanel(count_params.getEnd());
    startCommissionerTimePanel= new TimePanel(commis_params.getStart());
    endCommissionerTimePanel= new TimePanel(commis_params.getEnd());
    
    // commit buttons
    commitAdminStartButton= new Button("Commit");
    commitAdminEndButton= new Button("Commit");
    commitAnonymizerStartButton= new Button("Commit");
    commitAnonymizerEndButton= new Button("Commit");
    commitCounterStartButton= new Button("Commit");
    commitCounterEndButton= new Button("Commit");
    commitCommissionerStartButton= new Button("Commit");
    commitCommissionerEndButton= new Button("Commit");
    
    commitAdminStartButton.addActionListener(this);
    commitAdminEndButton.addActionListener(this);
    commitAnonymizerStartButton.addActionListener(this);
    commitAnonymizerEndButton.addActionListener(this);
    commitCounterStartButton.addActionListener(this);
    commitCounterEndButton.addActionListener(this);
    commitCommissionerStartButton.addActionListener(this);
    commitCommissionerEndButton.addActionListener(this);
    
    Panel timesPanel= new Panel();
    timesPanel.setLayout(new GridLayout(16,1));
    
    timesPanel.add(new Label("Admin Start", Label.CENTER));
    timesPanel.add(startAdminTimePanel);
    timesPanel.add(new Label("Admin End", Label.CENTER));
    timesPanel.add(endAdminTimePanel);
    timesPanel.add(new Label("Anonymizer Start", Label.CENTER));
    timesPanel.add(startAnonymizerTimePanel);
    timesPanel.add(new Label("Anonymizer End", Label.CENTER));
    timesPanel.add(endAnonymizerTimePanel);
    timesPanel.add(new Label("Counter Start", Label.CENTER));
    timesPanel.add(startCounterTimePanel);
    timesPanel.add(new Label("Counter End", Label.CENTER));
    timesPanel.add(endCounterTimePanel);
    timesPanel.add(new Label("Commissioner Start", Label.CENTER));
    timesPanel.add(startCommissionerTimePanel);
    timesPanel.add(new Label("Commissioner End", Label.CENTER));
    timesPanel.add(endCommissionerTimePanel);
    
    Panel commitPanel= new Panel();
    commitPanel.setLayout(new GridLayout(8,1));
    
    commitPanel.add(commitAdminStartButton);
    commitPanel.add(commitAdminEndButton);
    commitPanel.add(commitAnonymizerStartButton);
    commitPanel.add(commitAnonymizerEndButton);
    commitPanel.add(commitCounterStartButton);
    commitPanel.add(commitCounterEndButton);
    commitPanel.add(commitCommissionerStartButton);
    commitPanel.add(commitCommissionerEndButton);
    
    this.setLayout(new BorderLayout());
    this.add("Center", timesPanel);
    this.add("East",commitPanel);
    
  }

  public void actionPerformed(ActionEvent ae) {
    Object source= ae.getSource();

    TimePanel currentTP= null;
    Parameters currentParams= null;
    boolean is_start= true;

    if (source == commitAdminStartButton) {
      currentTP= startAdminTimePanel;
      currentParams= adminParams;
    }

    if (source == commitAdminEndButton) {
      currentTP= endAdminTimePanel;
      currentParams= adminParams;
      is_start= false;
    }

    if (source == commitAnonymizerStartButton) {
      currentTP= startAnonymizerTimePanel;
      currentParams= anonymizerParams;
    }

    if (source == commitAnonymizerEndButton) {
      currentTP= endAnonymizerTimePanel;
      currentParams= anonymizerParams;
      is_start= false;
    }
    
    if (source == commitCounterStartButton) {
      currentTP= startCounterTimePanel;
      currentParams= counterParams;
    }
    
    if (source == commitCounterEndButton) {
      currentTP= endCounterTimePanel;
      currentParams= counterParams;
      is_start= false;
    }

    if (source == commitCommissionerStartButton) {
      currentTP= startCommissionerTimePanel;
      currentParams= commissionerParams;
    }
    
    if (source == commitCommissionerEndButton) {
      currentTP= endCommissionerTimePanel;
      currentParams= commissionerParams;
      is_start= false;
    }
    
    if (is_start)
      currentParams.setStart(currentTP.getDate());
    else
      currentParams.setEnd(currentTP.getDate());
  }
}

/*
 * the class that presents a time
 */
class TimePanel extends Panel {

  /* GUI Elements */
  private TextField dayField;
  private java.awt.Choice monthChoice, ampmChoice;
  private TextField yearField;
  private TextField hourField, minuteField;

  public TimePanel(Date d) {
    this();
    
    if (d != null) {
      Calendar cal= Calendar.getInstance();
      cal.setTime(d);
      
      int day= cal.get(Calendar.DATE);
      int month= cal.get(Calendar.MONTH);
      int year= cal.get(Calendar.YEAR);
      int hour= cal.get(Calendar.HOUR);
      int minute= cal.get(Calendar.MINUTE);
      int amORpm= cal.get(Calendar.AM_PM);

      dayField.setText("" + day);
      yearField.setText("" + year);
      monthChoice.select(month);
      hourField.setText("" + hour);
      minuteField.setText("" + minute);
      if (amORpm == Calendar.AM)
	ampmChoice.select(0);
      else
	ampmChoice.select(1);
    }
    
  }

  public TimePanel() {
    
    dayField= new TextField(2);
    yearField= new TextField(4);

    monthChoice= new java.awt.Choice();
    monthChoice.add("January");
    monthChoice.add("February");
    monthChoice.add("March");
    monthChoice.add("April");
    monthChoice.add("May");
    monthChoice.add("June");
    monthChoice.add("July");
    monthChoice.add("August");
    monthChoice.add("September");
    monthChoice.add("October");
    monthChoice.add("November");
    monthChoice.add("December");

    hourField= new TextField(2);
    minuteField= new TextField(2);

    ampmChoice= new java.awt.Choice();
    ampmChoice.add("AM");
    ampmChoice.add("PM");

    this.add(monthChoice);
    this.add(dayField);
    this.add(new Label(","));
    this.add(yearField);
    this.add(new Label(" ; "));
    this.add(hourField);
    this.add(new Label(":"));
    this.add(minuteField);
    this.add(ampmChoice);

  }
  
  public Date getDate() {
    try {
      Calendar cal= Calendar.getInstance();
      cal.set(Calendar.YEAR, Integer.parseInt(yearField.getText()));
      cal.set(Calendar.MONTH, monthChoice.getSelectedIndex());
      cal.set(Calendar.DATE, Integer.parseInt(dayField.getText()));
      cal.set(Calendar.HOUR, Integer.parseInt(hourField.getText()));
      cal.set(Calendar.MINUTE, Integer.parseInt(minuteField.getText()));

      if (ampmChoice.getSelectedIndex() == 0)
	cal.set(Calendar.AM_PM, Calendar.AM);
      else
	cal.set(Calendar.AM_PM, Calendar.PM);

      Date returnDate= cal.getTime();
      return(returnDate);
    }
    catch(Exception exc) {
      System.out.println("got exception for date: " + exc.toString());
      exc.printStackTrace();
      return(null);
    }
  }
}
	   
