
/*
 * Disk Object Store
 *
 * stores objects in files on the disk
 * and retrieves them later
 */

package evote.store;

import java.io.*;
import java.util.*;

public class DiskObjectStore extends ObjectStore {
  
  private File directory;

  public DiskObjectStore() {
  }

  public void init(ObjectStoreParameters osp) throws ObjectStoreException {
    String dir= osp.getParameter("directory");

    if (dir == null)
      throw(new ObjectStoreException("No Directory Parameter Set"));

    directory= new File(dir);

    if (!directory.exists())
      throw(new ObjectStoreException("Directory does not exist"));

    if (!directory.isDirectory())
      throw(new ObjectStoreException("Not a Directory"));

    // find out what the maximum number is
    String[] files= directory.list();

    long fileNum;
    for (int i=0; i< files.length; i++) {
      try {
	fileNum= fileNameToNum(files[i]);
	
	updateKeyCounter(fileNum);
      } catch (Exception exc) {
	// not a proper file, no problem
	// we just keep going
      }
    }
  }

  private long fileNameToNum(String file) throws NumberFormatException {
    return(Long.parseLong(file));
  }

  private String numToFileName(long num) {
    return("" + num);
  }

  public synchronized void storeObject(long key, Object obj) throws ObjectStoreException {
    
    try {
      File storeFile= new File(directory, "" + numToFileName(key));

      FileOutputStream fos= new FileOutputStream(storeFile);

      ObjectOutputStream oos= new ObjectOutputStream(fos);

      oos.writeObject(obj);

      oos.close();
    } catch(Exception exc) {
      throw(new ObjectStoreException(exc.toString()));
    }
  }

  public Object loadObject(long key) throws ObjectStoreException {
    try {
      File storeFile= new File(directory, numToFileName(key));

      FileInputStream fis= new FileInputStream(storeFile);

      ObjectInputStream ois= new ObjectInputStream(fis);

      Object obj= ois.readObject();

      ois.close();

      return(obj);
    } catch(Exception exc) {
      throw(new ObjectStoreException(exc.toString()));
    }
  }

  public synchronized int getNumObjects() throws ObjectStoreException {
    int count=0;
    // find out what the maximum number is
    String[] files= directory.list();

    for (int i=0; i< files.length; i++) {
      try {
	long test= fileNameToNum(files[i]);
	
	count+= 1;
      } catch (Exception exc) {
	// not a proper file, no problem
	// we just keep going
      }
    }

    return(count);
  }

  public synchronized Enumeration loadAllObjects() throws ObjectStoreException {
    Vector objs= new Vector();
    Object current;

    for (int i=0; i<=keyCounter; i++) {
      try {
	current= loadObject(i);

	objs.addElement(current);
      } catch (Exception exc) {
      }
    }

    return(objs.elements());
  }

  public void destroy() {
  }
}
