package evote.util.genrand;

/* 
 *  GenRand.java
 *  Last Revised:  5 March 99
 *  Brandon DuRette <brandond@mit.edu>
 */

import evote.util.constants.*;
import evote.crypto.*;
//import java.lang.reflect.*;

/**
  The GenRand class generates cryptographically secure random numbers by 
  using a Generator to seed a SecureRandom (from Sun's JDK).  The 
  Generator is responsible for ensuring that the seed it uses is 
  irreproducible.  This will be done using either keyboard input,
  or mouse input.
  */

public class GenRand {
  
  private static String DEFAULT_GEN = "evote.util.genrand.MouseGenerator";
  // This is currently just a wrapper to java.util.Random.  I'm not sure
  // why.  I think it could be changed to a java.security.SecureRandom
  // at some point.
  private SecureRandom sr;	

  private Generator gen;	// Generates seeds for the random.
  
  public GenRand() {
    // effects:  Initialize a GenRand with the default Generator

    this(DEFAULT_GEN);
    System.out.println("Empty Constructor...");
  }

  public GenRand(String s) {
    // effects:  Initialize a GenRand witht the class found in s.
    //           On class not found exception, use the default.

    // Attempt to dynamically load the class.
    // On exception, instantiate a KeyboardGen.
    System.out.println("String Constructor...");
    try {
      Class       generatorClass       = Class.forName(s);

      gen = (Generator)(generatorClass.newInstance());
    } catch (IllegalAccessException e) {
      gen = new ThreadGenerator();
    } catch (InstantiationException e) {
      gen = new ThreadGenerator();
    } catch (ClassNotFoundException e) {
      gen = new ThreadGenerator();
    }
  }

  public void seed() {
    // effects:  seeds the generator using it's generator method.
    System.out.println("In seed method...");

    byte[] seed = gen.seed();
    System.out.println("Got seed.");
    sr = new SecureRandom(seed);
  }
  
  synchronized public byte[] nextBytes(int numBytes) {
    // effects:  returns the random data from the generator.
    byte randomBytes[] = new byte[numBytes];
    
    sr.nextBytes(randomBytes);
    return randomBytes;
  }
  
  public static void main(String argv[]) {

    
    GenRand g = new GenRand();
    g.seed();
  }
}

