/* Electronic Voting Project
 * Queue of Objects Class
 * written by Ben Adida (ben@mit.edu)
 */

package evote.util.queue;

public class Queue {
    private Object[] my_objects;
    private int total, start, size;
    private QueueFullException qfe;
    private QueueEmptyException qee;
    private boolean lock= false;
    
    public Queue(int num) {
	total= num;
	my_objects= new Object[num];
	start= 0;
	size= 0;
	qfe= new QueueFullException("Queue is Full: max=" +num);
	qee= new QueueEmptyException("Queue is Empty");
    }

    private int increase(int n) {
	n+= 1;
	if (n>= total)
	    n= 0;
	return(n);
    }

    private int decrease(int n) {
	n-= 1;
	if (n<0)
	    n= total-1;
	return(n);
    }

    private synchronized void getLock() {
	while(lock) {
	    try {
		Thread.sleep(50);
	    }
	    catch(Exception e) {}
	}
	lock= true;
    }

    private synchronized void releaseLock() {
	lock= false;
    }
    
    public boolean isFull() {
	if (size==total)
	    return(true);
	return(false);
    }

    public boolean isEmpty() {
	if (size==0)
	    return(true);
	return(false);
    }
    
    public int size() {
      return(size);
    }
	
    public synchronized void push(Object o) throws QueueFullException {
	getLock();
	if (this.isFull()) {
	    releaseLock();
	    throw(qfe);
	}
	my_objects[(start+size)%total]= o;
	size+= 1;
	releaseLock();
    }

    public synchronized Object pop() throws QueueEmptyException {
	Object o;
	getLock();
	if (this.isEmpty()) {
	    releaseLock();
	    throw(qee);
	}
	o= my_objects[start];
	start= increase(start);
	size -= 1;
	releaseLock();
	return(o);
    }

    public void clear() {
	start=0;
	size=0;
    }
}
