/*
 *  Shuffle.java
 *  Last Revised:  14 May 97
 *  Randy L. Milbert <rmilbert@mit.edu>
 */

package evote.util.shuffle;

import evote.util.genrand.*;

import java.applet.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.text.*;

public class Shuffle {
	public static final int bytes_per_int = 4;
	GenRand RNG;
	int num;

	public Shuffle (GenRand RNG, int num) {
		this.RNG = RNG;
		this.num = num;
	}

	public int[] get(int intArray[]) {
		byte randBytes[] = RNG.nextBytes(num * bytes_per_int);
		int randInts[] = new int[num];

		// concatenate 4 random bytes to create a single random int
		for (int i = 0; i < num; i++) {
			randInts[i] = randBytes[i*bytes_per_int] |
				(randBytes[i*bytes_per_int+1] << 8) |
				(randBytes[i*bytes_per_int+2] << 16) |
				(randBytes[i*bytes_per_int+3] << 24);
		}
		
		// shuffling:
		// go through the ints and randomly swap each one for another
		int temp;
		int swapElementIndex;
		for (int i = 0; i < num; i++) {
			swapElementIndex = abs(randInts[i]) % num;
			temp = intArray[i];
			intArray[i] = intArray[swapElementIndex];
			intArray[swapElementIndex] = temp;
		}

		return intArray;
	}

	public static void main(String S[]) {
		int num = 17;
		int order[] = new int[num];
		GenRand RNG = new GenRand();

		RNG.seed();

		for (int i = 0; i < num; i++)
			order[i] = i;

		for (int i = 0; i < num; i++)
			System.out.print(" "+order[i]);
		
		System.out.println("");

		
		Shuffle s = new Shuffle(RNG, num);
		order = s.get(order);

		for (int i = 0; i < num; i++)
			System.out.print(" "+order[i]);
	}

	private int abs(int num) {
		if (num < 0)
			return -num;
		return num;
	}
}


