//add comaplints

package evote.commissioner;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.security.*;
import java.util.Date;

import evote.commissioner.complaints.*;
import evote.crypto.*;
import evote.election.*;
import evote.network.message.*;
import evote.network.server.*;
import evote.registrar.*;
import evote.util.constants.*;
import evote.util.genrand.*;


/**
  The CommisionerServer is responsible for loggin all errors.  If there are
  any problems in the protocol, a connection will be opened to this server and
  a complaint will be sent to it (as well as logged by the complainer).
 */
public class CommissionerServer implements ConnectionHandler {
  Server my_server;
  String my_name = "Commissioner";
  int total_connection_count = 0;
  int total_confirmed_count = 0;
  int index = 0;
  RsaKeys commisKeys;
  public GenRand RNG;

/**
  This is the only constructor for the CommisionerServer.  Upon startup it
  creates a keyset for itself (e,d,n).
 */
  public CommissionerServer(Election elec) {

    Constants.init(elec);
    CommissionerParameters commisParams= elec.getCommissionerParameters();

    RNG = new GenRand();
    RNG.seed();
    String commisServer = Constants.commisServer;
    int commisReceivePort = Constants.commisReceivePort;
    BigInteger commisPublicKey = Constants.commisPublic;
    BigInteger commisPrivate = Constants.commisPrivate;
    BigInteger commisModulus = Constants.commisMod;
    try {
      commisKeys = new RsaKeys(commisPublicKey, commisPrivate, commisModulus);
    }
    catch(Exception exp) {
      System.out.println("Couldn't generate key set");
      System.exit(0);      
    }
    try {
      my_server= new SecureServer(my_name, Constants.commisReceivePort, this, Constants.commisNumThreads, Constants.commisQueueSize, commisKeys, RNG);
      my_server.setTimes(commisParams.getStart(), commisParams.getEnd());
      my_server.start();
    }
    catch(Exception exp) {
      System.err.println("Couldn't start up Commissioner server. Error: "+ exp.toString());
      exp.printStackTrace();
      System.exit(0);
    }
  }

/**
  Increments the count of the number of connections.
*/
  private synchronized void increase_connection_counter() {
    total_connection_count += 1;
  }

/**
  This is the complaint number used for reference.
*/
  private synchronized void increase_index() {
    index += 1;
  }

/**
  Increments the count of the number of complaints recorded.
*/
  private synchronized void increase_confirmed_counter() {
    total_confirmed_count += 1;
  }

/**
  This is the startup sequence.
*/
  public static void main(String argv[]) {
    DataInputStream din = new DataInputStream(System.in);
    DataOutputStream dout = new DataOutputStream(System.out);
    System.out.println("Starting commis server...");
    ElectionLoader eLoader= new ElectionLoader();
    new CommissionerServer(eLoader.getElection());
    System.out.println("started.");
  }

  public void handleConnection(Connection c) {
    try {
      increase_connection_counter();
      ComplaintMessage cm = (ComplaintMessage)c.receiveMessage();
      increase_index();
      Complaint comp = cm.getComplaint();
      comp.setTimeRcvd();
      String sender = cm.getSender();
      String recip = cm.getRecipient();
      String plantif = comp.getPlaintif();
      String defendant = comp.getDefendant();
      int error_no = comp.getProblemInt();
      String error_type = comp.getProblemString();
      String description = comp.getDescription();
      Date made_date = comp.getTimeMade();
      Date received_date = comp.getTimeReceived();
      my_server.log(new LogEntry(LogEntry.NORMAL_LOG,
				 "Complaint #" + index + ": " + "\n" +
				 "     Sender:        " + sender + "\n" +
				 "     Recipient:     " + recip + "\n" +
				 "     Plaintif:      " + plantif + "\n" +
				 "     Defendant:     " + defendant + "\n" +
				 "     Error No:      " + error_no + "\n" +
				 "     Error Type:    " + error_type + "\n" +
				 "     Description:   " + description + "\n" +
				 "     Time Made:     " + made_date.toString() + "\n" +
				 "     Time Received: " + received_date.toString() + "\n"));
      increase_confirmed_counter();
    }
    catch (Exception exp) {
      my_server.log(new LogEntry(LogEntry.WARNING_LOG, "Error with complaint number " + index + "\n" + exp.toString())); 
    }
  }

/**
  Method used to handle conncetion errors.
*/
  public void handleError(ConnectionError exp) {
      my_server.log(new LogEntry(LogEntry.WARNING_LOG, "Connection error with complaint number " + index + "\n" + exp.toString())); 
  }

  /* server shutting down */
  public void serverShutdown() {
  }

}






