/* Electronic Voting Project
 * Ben Adida(ben@mit.edu)
 * This is the class that performs the actual counting of votes
 */

package evote.counter;

import java.util.Enumeration;
import java.util.Vector;
import java.io.*;

import evote.lang.*;
import evote.counter.*;
import evote.election.*;
import evote.question.*;

/**
 * The Vote Counter
 *
 * Given a number of choices, this is the object that performs
 * the counting of the choices.
 *
 * @see VoteCounterDisplay
 */
public class MultipleVoteCounter {
  private Election my_election;
  private boolean votes_counted= false;
  private QuestionList[] questionLists;
  private ResultSet[] results;
  private Choice[] choices;
  
  /**
   * Create the Vote Counter
   * 
   * @argument e The election involved
   */
  public MultipleVoteCounter(Election e) {
    my_election= e;

    questionLists= e.getQuestionLists();
    results= new ResultSet[questionLists.length];
    for (int i=0; i<results.length; i++) {
      results[i]= new ResultSet(questionLists[i]);
    }
  }

  /**
   * Count the votes 
   *
   * @argument choice_enum The votes to count
   */
  public synchronized void countVotes(Enumeration vote_enum) {
    Vote currentVote;
    QuestionList ql;
    Question currentQuestion;
    MultipleChoice c;

    /* If Votes have already been counted, then don't count anymore*/
    if (votes_counted)
      return;
    
    votes_counted= true;

    while(vote_enum.hasMoreElements()) {
      try {
	currentVote= (Vote)vote_enum.nextElement();
	ql= currentVote.getQuestionList(); 
	c= (MultipleChoice)currentVote.getChoice();
      } catch (Exception exc) {
	System.out.println("VOTE COUNTER: Erroneous Vote");
	continue;
      }

      // we have a MultipleChoice object now
      processChoice(ql,c);
    }
  }

  private void processChoice(QuestionList ql, MultipleChoice mc) {
    Question thisQuestion;
    
    for (int i=0; i<ql.getNumQuestions(); i++) {
      thisQuestion= ql.getQuestion(i);

      try {
	try {
	  results[thisQuestion.getQuestionListID()].incrementWriteIn(new FullChoice(mc,ql), thisQuestion.getQuestionID(), mc.getWriteInChoice(i));
	} catch (ChoiceException exc) {
	  try {
	    results[thisQuestion.getQuestionListID()].incrementChoice(new FullChoice(mc,ql), thisQuestion.getQuestionID(), mc.getChoice(i));
	  } catch (WriteInException exc2) {
	  }
	}
      } catch (IndexOutOfBoundsException iobexc) {
	continue;
      }
    }
  }

  public ResultSet[] getResults() {
    return(null);
  }

  /**
   * Display the results of the counted votes
   *
   */
  public void displayResults() {
    if (!votes_counted)
      return;

    try {
      FileOutputStream fos= new FileOutputStream("RESULTS.txt");
      ObjectOutputStream oos= new ObjectOutputStream(fos);

      oos.writeObject(results);

      oos.close();
      fos.close();
    } catch (Exception exc) {
    }
  }
}
