/* Electronic Voting Project
 * Ben Adida(ben@mit.edu)
 * This is the class that performs the actual counting of votes
 */

package evote.counter;

import java.util.Enumeration;
import java.util.Vector;
import java.io.*;
import evote.lang.*;
import evote.counter.*;
import evote.election.*;
import evote.question.*;

/**
 * The Vote Counter
 *
 * Given a number of choices, this is the object that performs
 * the counting of the choices.
 *
 * @see VoteCounterDisplay
 */
public class SimpleMultipleVoteCounter {
  private Election my_election;
  private boolean votes_counted= false;
  
  /* The dimensions for vote_counts is [questionListNum][questionNum][choiceNum] */
  private int[][][] vote_counts;
  /* The dimensions for write_ins is [questionListNum][questionNum] */
  private Vector[][] write_ins;
  private QuestionList[] questionLists;
  private Question[][] questions;

  private int num_erroneous_votes=0;

  /**
   * Create the Vote Counter
   * 
   * @argument e The election involved
   */
  public SimpleMultipleVoteCounter(Election e) {
    my_election= e;
  }

  /**
   * Count the votes 
   *
   * @argument choice_enum The votes to count
   */
  public synchronized void countVotes(Enumeration vote_enum) {

    questionLists= my_election.getQuestionLists();

    String[][] votes = new String[questionLists.length][];
    questions= new Question[questionLists.length][];

    for (int i=0; i<votes.length; i++) {
      votes[i] = new String[questionLists[i].getNumQuestions()];
      questions[i]= new Question[questionLists[i].getNumQuestions()];
      for (int j=0; j<votes[i].length; j++) {
	votes[i][j]= "";
	questions[i][j]= questionLists[i].getQuestion(j);
      }
    }

    Vote currentVote;
    MultipleChoice currentChoice;
    QuestionList currentQL;
    Question currentQuestion;

    // go through all the votes!
    while (vote_enum.hasMoreElements()) {
      try {
	currentVote= (Vote)vote_enum.nextElement();
	currentQL= currentVote.getQuestionList();
	currentChoice= (MultipleChoice)currentVote.getChoice();
      } catch(Exception exc) {
	continue;
      }
   
      for (int i=0; i<currentChoice.getSize(); i++) {
	currentQuestion= currentQL.getQuestion(i);
	String[] answers= null;
	try {
	  answers= questions[currentQuestion.getQuestionListID()][currentQuestion.getQuestionID()].getAnswers();
	} catch(Throwable thr) {
	  answers= questions[0][0].getAnswers();
	}
	String oneVote= "";
	int this_choice;
	
	try {
	  for (int k=0; k<4; k++) {
	    try {
	      this_choice= currentChoice.getChoice(i);
	      // System.out.println("counting vote " + k + "," + this_choice);
	      oneVote+= "* " + answers[this_choice] + "\t";
	      if (answers[this_choice].length()<=13)
		oneVote+= "\t";
	      // System.out.println("done");
	    } catch (WriteInException wie) {
	      try {
		oneVote+= "* " +currentChoice.getWriteInChoice(i) + "\t";
		if (currentChoice.getWriteInChoice(i).length()<=13)
		  oneVote+= "\t";
	      } catch(Exception exc) {
		continue;
	      }
	    } 
	    currentChoice.downgradeQuestion(i);
	  }
	} catch(Throwable ioobe) {
	}

	try {
	  votes[currentQuestion.getQuestionListID()][currentQuestion.getQuestionID()] += oneVote + "\n";
	} catch (Throwable thr) {
	  votes[0][0]+= oneVote + "\n";
	}
      }
    }

    try {
      File resultFile= new File("RESULTS.txt");
      FileOutputStream fos= new FileOutputStream(resultFile);
      PrintStream resultStream= new PrintStream(fos);

      for (int i=0; i<votes.length; i++) {
	resultStream.println("\n\n == QUESTIONLIST #" +(i) + ": " + questionLists[i].getName()+"\n");
	for (int j=0; j<votes[i].length; j++) {
	  resultStream.println("\n - QUESTION #" + (j) + ": " + questionLists[i].getQuestion(j).getQuestion() + "\n");
	  
	  resultStream.println(votes[i][j]);
	}
      }
    } catch(Exception exc) {
      System.err.println("couldn't save results");
    }
    System.out.println("done");
  }

  /**
   * Display the results of the counted votes
   *
   */
  public void displayResults() {
    if (!votes_counted)
      return;
    
    new VoteCounterDisplay(my_election, vote_counts, write_ins);
  }
}
