/* Electronic Voting Project
 * Mark A. Herschberg (hershey@mit.edu)
 */

package evote.crypto;

import java.io.*;
import java.math.BigInteger;

/**
  Rsa private key consisting of two BigIntegers, d and n.
  */
public class PrivateKey implements EncryptionKey {
  BigInteger key;
  BigInteger modulus;
  
  /**
    Creates a PrivateKey object with the specified parameters.
    */
  public PrivateKey(BigInteger k, BigInteger m) {
    key = k;   
    modulus = m;
  }
  
  /**
    Gets a PrivateKey object from an byte array made with PrivateKey's
    toByteArray() method.
    */
  public static PrivateKey fromByteArray(byte[] b) throws Exception {
    try {
      ByteArrayInputStream bais= new ByteArrayInputStream(b);
      ObjectInputStream ois= new ObjectInputStream(bais);
      PrivateKey pk= (PrivateKey)ois.readObject();
      return(pk);
    }
    catch(Exception exp) {
      throw(new Exception("Format Error"));
    }
  }
  
  /**
    Returns the modulus n.
    */
  public BigInteger getModulus() {
    return modulus;
  }
  
  /**
    Returns the key d.
    */
  public BigInteger getKey() {
    return key;
  }
  
  /**
    Serializes the object to a byte array.
    */
  public byte[] toByteArray() {
    byte[] ser_pk;
    ByteArrayOutputStream baos= new ByteArrayOutputStream();
    try {
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(this);
      ser_pk= baos.toByteArray();
      return(ser_pk);
    }
    catch(IOException ie) {
      return null;
    }
  }
  
}





