/* Electronic Voting Project
 * Mark A. Herschberg (hershey@mit.edu)
 */

package evote.crypto;

import java.io.*;
import java.math.BigInteger;

/**
  An Rsa Public Key.
  */
public class RsaPublicKey implements EncryptionKey {
  BigInteger key;
  BigInteger modulus;
  
  public RsaPublicKey(BigInteger k, BigInteger m) {
    key = k;   
    modulus = m;
  }
  
  /**
    Gets a PublicKey object from an byte array made with PublicKey's
    toByteArray() mathod.
    */
  public static PublicKey fromByteArray(byte[] b) throws Exception {
    try {
      ByteArrayInputStream bais= new ByteArrayInputStream(b);
      ObjectInputStream ois= new ObjectInputStream(bais);
      PublicKey pk= (PublicKey)ois.readObject();
      return(pk);
    }
    catch(Exception exp) {
      throw(new Exception("Format Error"));
    }
  }

  /**
    Returns the modulus.
    */
  public BigInteger getModulus() {
    return modulus;
  }

  /**
    Returns the public key.
    */
  public BigInteger getKey() {
    return key;
  }

  /**
    Serializes the object to a byte array.
    */
  public byte[] toByteArray() {
    byte[] ser_pk;
    ByteArrayOutputStream baos= new ByteArrayOutputStream();
    try {
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(this);
      ser_pk= baos.toByteArray();
      return(ser_pk);
    }
    catch(IOException ie) {
      return null;
    }
  }
  
}

