/*
 * The Installer program for Elections
 * This installs the proper program for
 * the election loaded.
 */

package evote.installer;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

import evote.election.*;

public class Installer extends Frame implements ActionListener, ItemListener, WindowListener {

  private Button loadElectionButton, pickDirectoryButton, installButton;
  private TextField loadElectionField, pickDirectoryField, installField;
  private Choice serverChoice;
  private Panel mainPanel;
  private CardLayout myCardLayout;

  private Election myElection;

  private static final String VOTER_FILENAME= "Voter.election";
  private static final String ADMIN_FILENAME= "Admin.election";
  private static final String ANONYMIZER_FILENAME= "Anonymizer.election";
  private static final String COUNTER_FILENAME= "Counter.election";
  private static final String COMMISSIONER_FILENAME= "Commissioner.election";

  private static final String VOTER= "Voter Web Server";
  private static final String ADMIN_SERVER= "Administrator Server";
  private static final String ANONYMIZER_SERVER= "Anonymizer Server";
  private static final String COUNTER_SERVER= "Counter Server";
  private static final String COMMISSIONER_SERVER= "Commissioner Server";

  private String file, directory;
  
  public Installer() {
    super("Evox Installer");
    
    mainPanel= new Panel();
    myCardLayout= new CardLayout();
    mainPanel.setLayout(myCardLayout);
    
    Panel normalPanel= new Panel(new GridLayout(3,1));
    this.add("Center",mainPanel);
    mainPanel.add("NORMAL",normalPanel);
    myCardLayout.show(mainPanel, "NORMAL");

    // First Panel
    Panel firstPanel= new Panel(new BorderLayout());
    firstPanel.add("West", new Label("STEP #1"));
    
    Panel firstMainPanel= new Panel();
    loadElectionButton= new Button("Load Election File");
    loadElectionButton.addActionListener(this);
    loadElectionField= new TextField();
    loadElectionField.setEditable(false);
    firstMainPanel.add(loadElectionButton);
    
    firstPanel.add("Center",firstMainPanel);
    firstPanel.add("South",loadElectionField);

    // Second Panel
    Panel secondPanel= new Panel(new BorderLayout());
    secondPanel.add("West", new Label("STEP #2"));
    
    Panel secondMainPanel= new Panel();
    pickDirectoryButton= new Button("Pick Directory");
    pickDirectoryButton.addActionListener(this);
    pickDirectoryField= new TextField();
    pickDirectoryField.setEditable(false);
    serverChoice= new Choice();
    serverChoice.addItemListener(this);
    serverChoice.add(VOTER);
    secondMainPanel.add(pickDirectoryButton);
    secondMainPanel.add(serverChoice);

    secondPanel.add("Center",secondMainPanel);
    secondPanel.add("South",pickDirectoryField);
    
    // Third Panel
    Panel thirdPanel= new Panel(new BorderLayout());
    thirdPanel.add("West", new Label("STEP #3"));
    
    Panel thirdMainPanel= new Panel();
    installButton= new Button("Install Evox Server");
    installButton.addActionListener(this);
    installField= new TextField();
    installField.setEditable(false);
    thirdMainPanel.add(installButton);
    
    thirdPanel.add("Center",thirdMainPanel);
    thirdPanel.add("South",installField);

    // put it all together
    
    normalPanel.add(firstPanel);
    normalPanel.add(secondPanel);
    normalPanel.add(thirdPanel);
    
    // disable some buttons
    pickDirectoryButton.setEnabled(false);
    installButton.setEnabled(false);

    this.addWindowListener(this);
    
    this.pack();
    this.setSize(400,500);
    this.show();
  }

  public static void main(String argv[]) {
    new Installer();
  }

  public void itemStateChanged(ItemEvent ie) {
    installButton.setEnabled(false);
    pickDirectoryField.setText("Please select a directory and file");
  }

  public void actionPerformed(ActionEvent ae) {
    Object source= ae.getSource();

    /* The Button for loading an Election file is pressed */
    if (source == loadElectionButton) {
      try {
	FileDialog fd= new FileDialog(this);
	fd.show();
	String dir= fd.getDirectory();
	String file= fd.getFile();
	FileInputStream fis= new FileInputStream(new File(dir,file));
	ObjectInputStream ois= new ObjectInputStream(fis);
	myElection= (Election)ois.readObject();
	ois.close();

	serverChoice.removeAll();
	serverChoice.add(VOTER);

	// check which server this would be
	InetAddress localHost= InetAddress.getLocalHost();
	serverChoice.add(ADMIN_SERVER);
	serverChoice.add(COUNTER_SERVER);
	serverChoice.add(ANONYMIZER_SERVER);
	serverChoice.add(COMMISSIONER_SERVER);
	
	  
      } catch(Exception exc) {
	// display message that something is wrong
	System.out.println("couldn't open file:"+exc.toString());
	return;
      }

      // display election loaded and election hash
      loadElectionField.setText("Election Loaded");
      
      pickDirectoryButton.setEnabled(true);
      installButton.setEnabled(false);
    }

    /* If the pick directory button is pressed */
    if (source == pickDirectoryButton) {
      try {
	FileDialog fd= new FileDialog(this);
	String whichServer= serverChoice.getSelectedItem();
	if (whichServer.equals(ADMIN_SERVER))
	  fd.setFile(ADMIN_FILENAME);
	if (whichServer.equals(ANONYMIZER_SERVER))
	  fd.setFile(ANONYMIZER_FILENAME);
	if (whichServer.equals(COUNTER_SERVER))
	  fd.setFile(COUNTER_FILENAME);
	if (whichServer.equals(COMMISSIONER_SERVER))
	  fd.setFile(COMMISSIONER_FILENAME);
	if (whichServer.equals(VOTER))
	  fd.setFile(VOTER_FILENAME);

	fd.show();
	directory= fd.getDirectory();
	file= fd.getFile();
      }
      catch(Exception exc) {
	pickDirectoryField.setText("Error: " + exc.toString());
	return;
      }
	
      pickDirectoryField.setText("Directory: " + directory + " -- File: " + file);
      installButton.setEnabled(true);
    }

    /* If the install button is pressed */
    if (source == installButton) {
      
      Election newElection= (Election)myElection.clone();

      String whichServer= serverChoice.getSelectedItem();
      /* Prepare the election object */
      try {
	if (!whichServer.equals(ADMIN_SERVER)) {
	  newElection.getAdminParameters().getKeys().clearPrivate();
	  newElection.clearUsersAndQuestions();
	}
	
	if (!whichServer.equals(ANONYMIZER_SERVER))
	  newElection.getAnonymizerParameters().getKeys().clearPrivate();
	
	if (!whichServer.equals(COUNTER_SERVER))
	  newElection.getCounterParameters().getKeys().clearPrivate();
	
	if (!whichServer.equals(COMMISSIONER_SERVER))
	  newElection.getCommissionerParameters().getKeys().clearPrivate();
      } catch(Exception exc) {
	installField.setText("Error in Election File: " + exc.toString());
	exc.printStackTrace();
	return;
      }
	
      /* Write out the thing to file */
      try {
	FileOutputStream fos= new FileOutputStream(new File(directory,file));
	ObjectOutputStream oos= new ObjectOutputStream(fos);
	
	oos.writeObject(newElection);
      } catch(Exception exc) {
	installField.setText("Error: " + exc.toString());
	return;
      }

      installField.setText("Installation Complete");
    }
  }

  public void windowActivated(WindowEvent e) {}

  public void windowClosed(WindowEvent e) {}

  public void windowClosing(WindowEvent e) {System.exit(0);}
  public void windowDeactivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}

}
