/*
 * This is the Parameters object
 * It is used for storing parameters about each individual part of 
 * the voting system
 * This is the root for many other Parameter objects
 */

package evote.lang;

import evote.crypto.*;
import java.io.*;
import java.util.Date;

public class Parameters implements Serializable {

  private RsaKeys adminKeys, anonKeys, counterKeys, commisKeys;
  protected Date startDate, endDate;
  protected String hostname;
  protected int port;
  protected int numThreads, queueSize;

  public Parameters() {
  }

  public void setAdminKeys(RsaKeys rsk) {
    adminKeys= rsk;
  }
  
  public RsaKeys getAdminKeys() {
    return(adminKeys);
  }

  public void setAnonKeys(RsaKeys rsk) {
    anonKeys= rsk;
  }
  
  public RsaKeys getAnonKeys() {
    return(anonKeys);
  }

  public void setCounterKeys(RsaKeys rsk) {
    counterKeys= rsk;
  }
  
  public RsaKeys getCounterKeys() {
    return(counterKeys);
  }

  public void setCommisKeys(RsaKeys rsk) {
    commisKeys= rsk;
  }

  public RsaKeys getCommisKeys() {
    return(commisKeys);
  }

  public void setKeys(RsaKeys rsk) {
  }
  
  public RsaKeys getKeys() {
    return(null);
  }
  
  public void clearPrivate() {
    try {
      adminKeys.clearPrivate();
      counterKeys.clearPrivate();
      anonKeys.clearPrivate();
      commisKeys.clearPrivate();
    } catch(NullPointerException exc) {
    }
  }
	
  public void setStart(Date start) {
    startDate= start;
  }

  public void setEnd(Date end) {
    endDate= end;
  }
  
  public Date getStart() {
    return(startDate);
  }

  public Date getEnd() {
    return(endDate);
  }
  
  public void setNumThreads(int num_threads) {
    numThreads= num_threads;
  }
  
  public int getNumThreads() {
    return(numThreads);
  }

  public void setQueueSize(int queue_size) {
    queueSize= queue_size;
  }

  public int getQueueSize() {
    return(queueSize);
  }
  
  public void setLocation(String hn, int port) {
    hostname= hn;
    this.port= port;
  }
  
  public String getHostName() {
    return(hostname);
  }

  public int getPort() {
    return(port);
  }

  public Parameters makeOtherSelf() {
    return(new Parameters());
  }

  public Object clone() {
    Parameters otherSelf= this.makeOtherSelf();

    if (this.adminKeys != null)
      otherSelf.adminKeys= (RsaKeys)this.adminKeys.clone();
    
    if (this.anonKeys != null)
      otherSelf.anonKeys= (RsaKeys)this.anonKeys.clone();

    if (this.counterKeys != null)
      otherSelf.counterKeys= (RsaKeys)this.counterKeys.clone();

    if (this.commisKeys != null)
      otherSelf.commisKeys= (RsaKeys)this.commisKeys.clone();

    otherSelf.startDate= this.startDate;
    otherSelf.endDate= this.endDate;
    otherSelf.hostname= this.hostname;
    otherSelf.port= this.port;
    otherSelf.numThreads= this.numThreads;
    otherSelf.queueSize= this.queueSize;

    return(otherSelf);
  }

}
