/* Electronic Voting Project
 * This is the listener to server connection events
 * Ben Adida (ben@mit.edu)
 */

package evote.network.server;

import java.util.EventListener;
import java.util.EventObject;
import evote.network.server.ConnectionEvent;

/**
 * Classes that implement this interface can receive information
 * from the Server concerning new connections, ended connections,etc..
 */
public interface ConnectionListener extends EventListener {
  /**
   * This is called when a connection is started
   * 
   * @argument ce The connection event that describes what happened.
   */
  public void connectionStarted(ConnectionEvent ce);

  /**
   * This is called when a connection is ended.
   * 
   * @argument ce The connection event that describes what happened.
   */
  public void connectionEnded(ConnectionEvent ce);

  /**
   * This is called when a connection is queued
   * 
   * @argument ce The connection event that describes what happened.
   */
  public void connectionQueued(ConnectionEvent ce);

  /**
   * This is called when a connection is dequeued
   * 
   * @argument ce The connection event that describes what happened.
   */
  public void connectionDequeued(ConnectionEvent ce);
}
