/*
 * ElectionViewer.java
 * Last Revised:  13 May 97
 * Randy L. Milbert <rmilbert@mit.edu>
 */

package evote.question;

import evote.util.quick_message.QuickMessage;
import evote.election.*;
import evote.lang.*;
import evote.voter.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 *  The ElectionViewer panel provides an interface that the voter
 *  may use to make his or her selections.
 */

public class QuestionListViewer extends Panel {

	private	QuestionList			election;
	private	QuestionPanel		question_panel[];
	private	int					num_questions;
	private	evote.lang.Choice				choice;
  private Booth parent;

  /**
   *  The constructor takes three objects as arguments:  Dimension, Election,
   *  and Booth.  The Dimension object specifies the height and width of the
   *  panel.  The Election object contains the questions and choices of the
   *  ballot.  The Booth object is notified when all of the selections have
   *  been made.
   */

  public QuestionListViewer(QuestionList election, Booth parent) {
    
    num_questions		= election.getNumQuestions();
    this.parent = parent;
    
    // setSize(win_dimension);
    setLayout(new CardLayout());
    setBackground(Color.lightGray);
    
    choice = new evote.lang.Choice(num_questions);
    
    question_panel = new QuestionPanel[num_questions];
    
    for (int i = 0; i < num_questions; i++) {
      question_panel[i] = new QuestionPanel(this, election, choice, i);
    }
  }
  
  public QuestionListViewer(QuestionList election, Booth parent, evote.lang.Choice ch) {
    num_questions		= election.getNumQuestions();
    this.parent = parent;
    
    // setSize(win_dimension);
    setLayout(new CardLayout());
    setBackground(Color.lightGray);
    
    choice= ch;
    
    question_panel = new QuestionPanel[num_questions];
    
    for (int i = 0; i < num_questions; i++) {
      // HACK!
      if (ch instanceof MultipleChoice)
	question_panel[i] = new MultipleQuestionPanel(this, election, (MultipleChoice)choice, i);
      else
	question_panel[i] = new QuestionPanel(this, election, choice, i);
    }
  }    

  /**
   *  The getChoice() method returns the Choice object generated by the
   *  user's selections.
   */

  public evote.lang.Choice getChoice() {
    return choice;
  }

  public void setStatus(String st) {
    parent.setStatus(st);
  }

  /**
   *  The done() method is called when the user is finished making his
   *  or her selections.
   */

  public void done() {
    parent.done();
  }
}


