/*
 * QuestionPanel.java
 * Last Revised:  13 May 97
 * Randy L. Milbert <rmilbert@mit.edu>
 */

package evote.question;

import evote.util.quick_message.QuickMessage;
import evote.election.*;
import evote.lang.Choice;

import java.awt.*;
import java.awt.event.*;

/**
 *  The QuestionPanel panel displays one question from the election.
 *  Whether or not the question allows write-ins is displayed as are
 *  the valid choices.
 */

public class QuestionPanel extends Panel implements ActionListener {
	private	CheckboxGroup	check_box_group;
	private	Checkbox		check_box[];
	private	Checkbox		write_in_check_box;
	private	Label			question_label;
	private	Label			write_in_label;
	private	TextField		write_in_text_field;
	private	Button			prev_button;
	private	Button			next_button;
	private	Button			done_button;
	private	int				num_question;
	private	int				num_choices;
  private Panel grandparent;
	private	QuestionListViewer	       	parent;
	private	QuestionList		election;
	private	Choice			choice;

  /**
   *  The constructor takes five objects as arguments.  The ElectionViewer
   *  panel necessary for CardLayout.  The Dimension object specifies the
   *  height and width of the panel.  The Election object contians the
   *  text of the question and the text of the choices.  The choice object
   *  is altered when the user makes his or her selections.  The num_question
   *  variable is the question number of the election that this panel is
   *  responsible for.
   */

  protected QuestionPanel() {
  }

	public QuestionPanel(QuestionListViewer parent, QuestionList election, Choice choice, int num_question) {
	  this.parent			= parent;
	  this.election		= election;
	  this.num_question	= num_question;
	  this.choice			= choice;
	  
	  // setSize(win_dimensions);
	  setBackground(Color.white);
	  
	  Panel question_panel= new Panel();
	  Panel bottom_panel= new Panel();
	  
	  Question currentQuestion= election.getQuestion(num_question);
	  int num_rows= currentQuestion.getNumAnswers();
	  if (currentQuestion.allowsWriteIns())
	    num_rows+= 1;
	  question_panel.setLayout(new GridLayout(num_rows,1));
	  this.setLayout(new BorderLayout());
	  this.add("Center", question_panel);
	  bottom_panel.setLayout(new FlowLayout());
		
	  num_choices			= currentQuestion.getNumAnswers();
	  check_box			= new Checkbox[num_choices+1];
	  
	  Panel topPanel= new Panel();
	  TextArea question_area= new TextArea("Question #" + (num_question+1) + "\n" + currentQuestion.getQuestion(),8,75,TextArea.SCROLLBARS_NONE);
	  question_area.setEditable(false);
	  topPanel.add(question_area);
		
	  check_box_group = new CheckboxGroup();
	  String[] answers= currentQuestion.getAnswers();
	  for (int i = 0; i < num_choices; i++) {
	    check_box[i]	= new Checkbox(answers[i], check_box_group, false);
	    question_panel.add(check_box[i]);
	  }
	  
	  if (currentQuestion.allowsWriteIns()) {
	    Panel write_in_panel= new Panel();
	    write_in_check_box = new Checkbox("Write-in:", check_box_group, false);
	    write_in_panel.add(write_in_check_box);
	    write_in_text_field	= new TextField(28);
	    write_in_panel.add(write_in_text_field);
	    question_panel.add(write_in_panel);
	  }
	  
	  if (num_question != 0) {
	    prev_button	= new Button(" < Previous");
	    bottom_panel.add(prev_button);
	    prev_button.addActionListener(this);
	  }
	  
	  if (num_question == election.getNumQuestions() - 1) {
	    done_button	= new Button("VOTE");
	    bottom_panel.add(done_button);
	    done_button.addActionListener(this);
	  }
	  else {
	    next_button	= new Button("Next > ");
	    bottom_panel.add(next_button);
	    next_button.addActionListener(this);
	  }
	  
	  this.add("North",topPanel);
	  this.add("South",bottom_panel);
	  parent.add("QuestionPanel " + num_question, this);
	}

	public void actionPerformed(ActionEvent event) {
		Object source = event.getSource();
		
		if (source == next_button || source == done_button) {
			Checkbox response = check_box_group.getSelectedCheckbox();

			if (response == null) {
			  parent.setStatus("Error: No Choice Specified");
			  // new QuickMessage("Error:  No Choice Specified","Error:  no choice was specified.");
				return;
			}
			parent.setStatus("");
			
			if (write_in_check_box != null && response == write_in_check_box) {
				if (write_in_text_field.getText().length() == 0) {
				  new QuickMessage("Error:  No Write-In Specified", "Error:  although the write-in option was specified, the write-in itself was not.");
					return;
				}
			}

			if (response == write_in_check_box) {
				choice.setChoiceWriteIn(num_question + 1, write_in_text_field.getText());
			}
			else {
				for (int i = 0; i < num_choices; i++) {
					if (response == check_box[i]) {
						choice.setChoice(num_question + 1, i + 1);
						break;
					}
				}
			}
			// System.out.println(choice.toString());

			if (source == next_button)
				((CardLayout)parent.getLayout()).next(parent);

			if (source == done_button) {
			  parent.done();
			}
		}

		else if (source == prev_button) {
			((CardLayout)parent.getLayout()).previous(parent);
		}
	}
}
