
/*
 * This class is an object factory for subclasses of ObjectStore.
 * It provides a common interface for permanent storage of Serializable
 * objects. Each object is stored with a unique key (which is a long).
 * The key must be unique per store, and a store provides a way of generating 
 * a valid unique key.
 */

package evote.store;

import java.util.Enumeration;

public class ObjectStore {
  
  protected long keyCounter=0;
  
  // We don't allow basic ObjectStores to be create
  protected ObjectStore() {
  }

  // we allow ObjectStores to be created through this
  // factory-like creation
  public static ObjectStore newInstance(String type) throws InstantiationException {
    try {
      Class storeClass= Class.forName(type);

      ObjectStore objStore= (ObjectStore) storeClass.newInstance();

      return(objStore);
    } catch(Exception exc) {
      throw(new InstantiationException(exc.toString()));
    }
  }

  public void init(ObjectStoreParameters osp) throws ObjectStoreException {
  }

  public synchronized long getNextKey() {
    keyCounter+= 1;
    return(keyCounter);
  }

  // update the Key Counter 
  // usually when starting up an ObjectStore
  // and initializing all the data
  protected synchronized void updateKeyCounter(long num) {
    if (keyCounter <= num)
      keyCounter= num;
  }

  public void storeObject(long key, Object obj) throws ObjectStoreException {
  }

  public Object loadObject(long key) throws ObjectStoreException {
    return(null);
  }

  public int getNumObjects() throws ObjectStoreException {
    return(0);
  }

  public Enumeration loadAllObjects() throws ObjectStoreException {
    return(null);
  }

  public void destroy() {
  }
}
